import unittest, random
if __name__ == '__main__':
    import os, sys
    sys.path.append( os.path.dirname( 
            os.path.dirname( os.path.abspath( __file__ ) ) ) )

from mivrhashclient import MivRHash


class TestRHash( unittest.TestCase ):
    def setUp( self ):
        self.m_h = MivRHash( "localhost", 10080 )

    def tearDown( self ):
        self.m_h = None
    
    def test_insert( self ):
        i = 0
        while i < 10000:
            self.m_h[i] = i + 1
            i += 1
        i = 0
        while i < 10000:
            j = self.m_h[i]
            self.assertEqual( j, i + 1 )
            i += 1

    def test_delete( self ):
        i = 0
        while i < 10000:
            self.m_h[i] = random.random()
            i += 1
        print "Inserted 10000 entries"
        i = 0
        while i < 1000:
            self.assertTrue( self.m_h.has_key( i ) )
            del self.m_h[i]
            self.assertFalse( self.m_h.has_key( i ) )
            i += 1
            if i % 100 == 0:
                print "Deleted %d entries" % i 

    def test_clear( self ):
        i = 0
        while i < 10000:
            self.m_h[ random.random() ] = random.random()
            i += 1
        self.assertTrue( len( self.m_h ) > 0 )
        self.m_h.clear()
        self.assertEqual( len( self.m_h ),  0 )

    def test_random_access( self ):
        i = 0
        while i < 10000:
            self.m_h[i] = i + 1
            i += 1
        indexLst = range( 0, 10000 )
        random.shuffle( indexLst )
        count = 0
        for i in indexLst:
            count += 1
            self.assertEqual( self.m_h[i], ( i + 1 ) )
            if count % 1000 == 0:
                print "Accessed %d/10000 entries" % count

    def test_random_delete( self ):
        i = 0
        while i < 2500:
            self.m_h[i] = i + 1
            i += 1
        indexLst = range( 0, 2500 )
        random.shuffle( indexLst )
        count = 0
        for i in indexLst:
            count += 1
            self.assertTrue( self.m_h.has_key( i ) )
            del self.m_h[i]
            self.assertFalse( self.m_h.has_key( i ) )
            if count % 100 == 0:
                print "Deleted %d/2500 entries" % count

    def test_random_hash( self ):
        i = 0
        while i < 10000:
            self.m_h[i] = i + 1
            i += 1
        indexLst = range( 0, 10000 )
        random.shuffle( indexLst )
        for i in range( 0, 200 ):
            j = random.randint( 0, 9999 )
            # this is slightly different from dict: nothing raises!
            del self.m_h[j]
            self.assertFalse( self.m_h.has_key( j ) )
        for i in range( 0, 2000 ):
            j = random.randint( 0, 99999 )
            self.m_h[j] = j - 1
        i = 0
        while i < 10000:
            if self.m_h.has_key( i ):
                v = self.m_h[i]
                self.assertTrue( ( ( v == i + 1 ) or ( v == i - 1 ) ) )
            i += 1
            if i % 1000 == 0:
                print "Accessed %d/10000 entries" % i
        
if __name__ == '__main__':
    unittest.main()

        
    



"""
  The server side for Miv Remote Hash Table
  Usage: python mivrhashserver.py [nPort]
  Default port is 10080
  (C) 2010, Duc <duc at mi.ci.i.u-tokyo.ac.jp>
"""

import sys, os, socket, threading, thread, urllib, cPickle

class RsHashService:
    """
    Remote Hash service: listen to connection from clients and create hashes.

    Each client is served in a separated thread.
    """
    def __init__( self ):
        self.m_childrenList = {}
        self.m_childrenListLock = threading.Lock()
        pass
    def serve_client( self, conn, addr ):
        self.m_childrenListLock.acquire()
        self.m_childrenList[ thread.get_ident() ] = threading.currentThread()
        self.m_childrenListLock.release()
        print "Accepted: %s:%d" % addr
        fp = conn.makefile()
        fp.write( "RsHashService %d\n" % addr[1] )
        fp.flush()
        strLine = fp.readline().strip()
        if strLine != ( "RsHashClient %d" % addr[1] ):
            fp.write( "Invalid Protocol" )
            fp.flush()
            fp.close()
            conn.close()
        else:
            myHash = {}
            myIter = None  # iterator for the hash
            print "Created a new hash for %s:%d" % addr
            while True:
                strLine = fp.readline().strip()
                strLine = urllib.unquote( strLine )
                argLst = cPickle.loads( strLine )
                if len( argLst ) <= 0:
                    continue
                cmd = argLst[0]
                if ( cmd == 'get' ) or ( cmd == 'has_key' ):
                    if len( argLst ) <= 1:
                        continue
                    theKey = argLst[1]
                    if myHash.has_key( theKey ):
                        responseObj = ( True, myHash[theKey] )
                    else:
                        responseObj = ( False, None )
                    strResponse = cPickle.dumps( responseObj )
                    strResponse = urllib.quote( strResponse ) + "\n"
                    fp.write( strResponse )
                    fp.flush()
                elif cmd == 'put':
                    if len( argLst ) <= 2:
                        continue
                    theKey = argLst[1]
                    theVal = argLst[2]
                    myHash[theKey] = theVal
                elif cmd == 'batchput':
                    if len( argLst ) <= 1:
                        continue
                    putPairs = argLst[1]
                    for aPair in putPairs:
                        myHash[aPair[0]] = aPair[1]
                elif cmd == 'size':
                    strResponse = cPickle.dumps( len( myHash ) )
                    strResponse = urllib.quote( strResponse ) + "\n"
                    fp.write( strResponse )
                    fp.flush()
                elif cmd == 'init_iter':
                    if len( argLst ) <= 1:
                        continue
                    iterSubject = argLst[1]
                    if iterSubject == 'keys':
                        myIter = myHash.iterkeys()
                    elif iterSubject == 'values':
                        myIter = myHash.itervalues()
                    elif iterSubject == 'items':
                        myIter = myHash.iteritems()
                elif cmd == 'iter':
                    if len( argLst ) <= 1:
                        continue
                    chunkLen = argLst[1]
                    nIterCount = 0
                    retLst = []
                    bStopIter = False
                    while nIterCount < chunkLen:
                        try:
                            theItem = myIter.next()
                            retLst.append( theItem )
                            nIterCount += 1
                        except StopIteration:
                            bStopIter = True
                            myIter = None
                            break
                    retObj = ( bStopIter, retLst )
                    strResponse = cPickle.dumps( retObj )
                    strResponse = urllib.quote( strResponse ) + "\n"
                    fp.write( strResponse )
                    fp.flush()
                elif cmd == 'clear':
                    myHash = {}
                elif cmd == 'del':
                    if len( argLst ) <= 1:
                        continue
                    theKey = argLst[1]
                    if myHash.has_key( theKey ):
                        del myHash[theKey]
                elif cmd == 'close':
                    myHash = None
                    fp.close()
                    break
        conn.close()
        self.m_childrenListLock.acquire()
        if self.m_childrenList.has_key( thread.get_ident() ):
            del self.m_childrenList[ thread.get_ident() ]
        self.m_childrenListLock.release()
        print "%s:%d disconnected" % addr
        
    def start( self, port ):
        """
        Start listen on port port.
        """
        s = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
        s.setsockopt( socket.SOL_SOCKET, socket.SO_REUSEADDR, 1 )
        s.bind( ( '', port ) )
        s.listen( 5 )
        print "Server listening on port %d" % port
        while True:
            conn, addr = s.accept()
            newTh = threading.Thread( group = None, target = self.serve_client,
                                      name = None,
                                      args = ( conn, addr ),
                                      kwargs = {} )
            newTh.start()

    def start_service( self, port = 10080 ):
        try:
            self.start( port )
        except KeyboardInterrupt:
            # wait for all children to exit
            self.m_childrenListLock.acquire()
            for aTh in self.m_childrenList.itervalues():
                try:
                    aTh.join()
                except:
                    pass
            self.m_childrenListLock.release()
        print "Server terminated"

if __name__ == "__main__":
    def rs_hash_main():
        if len( sys.argv ) > 1:
            try:
                nPort = int( sys.argv[1] )
            except:
                print "Usage: python mivrhashservice.py [nPort]"
                print "Default port number is 10080"
                return
        else:
            nPort = 10080
        svr = RsHashService()
        svr.start_service( nPort )
    # invoke the main
    rs_hash_main()

            
    
            



from distutils.core import setup

setup( name = 'mivrhash',
       version = '1.0.1',
       author = 'Conbodien',
       author_email = 'duc@mi.ci.i.u-tokyo.ac.jp',
       url = 'http://pypi.python.org/pypi/mivrhash/',
       packages = [ 'mivrhash', 'mivrhash.testsuite' ],
       description = 'A hash table (Python dict compatible) that stores data in remote servers. Performance is optimized by using write-back cache.',
       long_description = open( 'README.txt' ).read(),
       classifiers = [
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Topic :: Utilities',
          'Topic :: System :: Distributed Computing',
          'License :: OSI Approved :: Python Software Foundation License'
        ],
       keywords = 'remote hash, hash table',
       platforms = [ 'Any' ],
       )




"""
  The client side for Miv Remote Hash Table.
  Usage:
    rHash = MivRHash( "server_address", nPort )
    rHash[key] = value
    (all functions are similar to Python dict)
  (C) 2010 by Duc <duc at mi.ci.i.u-tokyo.ac.jp>
"""

import sys, os, socket, traceback, exceptions, urllib, cPickle
from operator import itemgetter

class RsHashClient:
    def __init__( self ):
        self.m_connected = False
        pass
    def connect( self, strHost, nPort ):
        """
        Connect to the server and create a new hash on it.
        """
        try:
            self.m_sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
            self.m_sock.connect( ( strHost, nPort ) )
            self.m_fp = self.m_sock.makefile()
            strAck = self.m_fp.readline()
            strAck = strAck.replace( "RsHashService", "RsHashClient" )
            self.m_fp.write( strAck )
            self.m_fp.flush()
        except Exception:
            traceback.print_exc()
            return False
        self.m_connected = True
        return True

    def put( self, theKey, theVal ):
        if not self.m_connected:
            raise exceptions.IOError( 'Put', 'RsHashClient not connected' )
        objToSend = ( 'put', theKey, theVal )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        # self.m_fp.readline()

    def batch_put( self, pairLst ):
        """
        Put many (key, value) pair to the remote server.
        
        This is for reducing the time for write_back.
        """
        if not self.m_connected:
            raise exceptions.IOError( 'BatchPut', 'RsHashClient not connected' )
        objToSend = ( 'batchput', pairLst )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        
    def get( self, theKey ):
        if not self.m_connected:
            raise exceptions.IOError( 'Get', 'RsHashClient not connected' )
        objToSend = ( 'get', theKey )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        strRet = self.m_fp.readline().strip()
        objRecv = cPickle.loads( urllib.unquote( strRet ) )
        if not objRecv[0]:
            raise exceptions.KeyError( 'RsHashGet', theKey )
        theVal = objRecv[1]
        return theVal

    def get_size( self ):
        if not self.m_connected:
            raise exceptions.IOError( 'Size', 'RsHashClient not connected' )
        objToSend = ( 'size', )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        strRet = self.m_fp.readline().strip()
        objRecv = cPickle.loads( urllib.unquote( strRet ) )
        return objRecv
    
    def remove( self, theKey ):
        """
        Remove theKey from this hash.
        """
        if not self.m_connected:
            raise exceptions.IOError( 'Remove', "RsHashClient not connected" )
        objToSend = ( 'del', theKey )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()

    def clear( self ):
        """
        Clear all data in the hash.
        """
        if not self.m_connected:
            raise exceptions.IOError( 'Remove', "RsHashClient not connected" )
        objToSend = ( 'clear', )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()

    def get_iter( self, subj ):
        """
        Start an iterator on remote server.

        Many get_iter_chunk can be called after this call.
        """
        if not self.m_connected:
            raise exceptions.IOError( 'GetIter', 'RsHashClient not connected' )
        objToSend = ( 'init_iter', subj )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        self.m_bInIter = True

    def get_iter_chunk( self, nMaxLen ):
        """
        Get the next chunk of data for iterating the hash.
        """
        # a get_iter must be called before this
        objToSend = ( 'iter', nMaxLen )
        strToSend = cPickle.dumps( objToSend )
        strToSend = urllib.quote( strToSend ) + "\n"
        self.m_fp.write( strToSend )
        self.m_fp.flush()
        strRet = self.m_fp.readline().strip()
        objRecv = cPickle.loads( urllib.unquote( strRet ) )
        return objRecv
    
    def close( self ):
        """
        Free memory and close the connection to remote server.
        """
        if self.m_connected:
            objToSend = ( 'close', )
            strToSend = cPickle.dumps( objToSend )
            strToSend = urllib.quote( strToSend ) + "\n"
            self.m_fp.write( strToSend )
            self.m_fp.flush()
            self.m_fp.close()
            self.m_sock.close()
            self.m_connected = False
            
    def __del__( self ):
        if self.m_connected:
            import cPickle, urllib
            objToSend = ( 'close', )
            strToSend = cPickle.dumps( objToSend )
            strToSend = urllib.quote( strToSend ) + "\n"
            self.m_fp.write( strToSend )
            self.m_fp.flush()
            self.m_fp.close()
            self.m_sock.close()
            self.m_connected = False
    

class MivRHashIter:
    """
    Iterator for MivRHash
    """
    def __init__( self, iterType, theHash ):
        self.m_iterType = iterType
        self.m_nCacheStartIndex = -1
        self.m_nStartIndex = 0
        self.m_cachedItems = None
        self.m_h = theHash
        self.m_bWillStop = False

    def __iter__( self ):
        return self

    def next( self ):
        iterType = self.m_iterType
        if self.m_iterType != 'values':
            iterType = 'items'
        if self.m_nCacheStartIndex < 0:
            self.m_h.m_rsHashClient.get_iter( iterType )
        if ( not self.m_cachedItems ) or (
            ( self.m_nCacheStartIndex + len( self.m_cachedItems
                                             ) <= self.m_nStartIndex ) and (
                 not self.m_bWillStop
               ) ):
            retObj = self.m_h.m_rsHashClient.get_iter_chunk(
                self.m_h.m_nCacheSize / 2 )
            self.m_cachedItems = retObj[1]
            self.m_bWillStop = retObj[0]
            self.m_nCacheStartIndex = self.m_nStartIndex
            if iterType == 'items':
                self.m_h.batch_set_item( self.m_cachedItems )
        if self.m_nStartIndex >= self.m_nCacheStartIndex + len( self.m_cachedItems ):
            raise StopIteration()
        theIndex = self.m_nStartIndex - self.m_nCacheStartIndex
        self.m_nStartIndex += 1
        if self.m_iterType == 'keys':
            return self.m_cachedItems[theIndex][0]
        else:
            return self.m_cachedItems[theIndex]
        
                 
        
class MivRHash:
    """
    Remote hash table, provide the same interface with the Python dict
    
    The actual data of the hash table may be stored in a remote host
    """
    def __init__( self, strHost, nPort, nCacheSize = 1000 ):
        """
        Create a new MivRHash, store data on host strHost
        """
        self.m_nCacheSize = nCacheSize
        self.m_cachedItems = {}
        self.m_nMaxCacheId = 0
        # cacheUpdateTime update threshold: history is saved in 28 bits
        self.m_nCacheUpdateThreshold = self.m_nCacheSize / 28
        # bit index starts from 0
        # bit 30: Not put yet, bit 29: Dirty, bit 28: Reference 
        # bit 27-0: reference history
        self.m_cacheUpdateTime = {}
        self.m_rsHashClient = RsHashClient()
        self.m_rsHashClient.connect( strHost, nPort )

    def update_cache_info( self ):
        # update the cacheUpdateTime table by aging the bits
        for cacheId in self.m_cacheUpdateTime.iterkeys():
            theTime = self.m_cacheUpdateTime[cacheId]
            nU = ( theTime & 0x0FFFFFFF )  # least 28 bits
            nRD = ( ( theTime >> 28 ) & 0x01 )  # Reference bit
            nU = ( ( nU >> 1 ) | ( nRD << 27 ) )  # new value for nU
            theTime = ( ( theTime & 0x60000000 ) | nU )
            self.m_cacheUpdateTime[cacheId] = theTime
        self.m_nCacheUpdateThreshold = self.m_nCacheSize / 28

    def insert_item( self, theKey, theVal, bDirty ):
        # insert an item into the cache
        if self.m_nMaxCacheId >= ( 1 << 30 ):
            self.refresh_cache_id()
        self.m_nMaxCacheId += 1
        cacheId = self.m_nMaxCacheId
        self.m_cachedItems[theKey] = ( cacheId, theVal )
        theTime = ( 1 << 28 )
        if bDirty:
            # set the NotPut bit and the Dirty bit
            theTime |= ( ( 1 << 30 ) | ( 1 << 29 ) )
        self.m_cacheUpdateTime[cacheId] = theTime
        
    def do_write_back( self ):
        # write dirty items back to the server
        pairList = []
        for theKey, thePair in self.m_cachedItems.iteritems():
            cacheId, theVal = thePair
            theTime = self.m_cacheUpdateTime[cacheId]
            if ( ( theTime >> 29 ) & 0x01 ) > 0:  # if dirty
                # self.m_rsHashClient.put( theKey, theVal )
                pairList.append( ( theKey, theVal ) )
                # clear the NotPut bit and Dirty bit
                theTime &= 0x1FFFFFFF
                self.m_cacheUpdateTime[cacheId] = theTime
        self.m_rsHashClient.batch_put( pairList )
        
    def do_swap_out( self ):
        # swap out the LRU (NRU) items
        self.do_write_back()
        sortedCacheIdList = sorted( self.m_cacheUpdateTime.items(),
                                    key = itemgetter( 1 ) )
        nToSwapOut = len( sortedCacheIdList ) / 2
        swapOutSet = set()
        for itm in sortedCacheIdList:
            cacheId, theTime = itm
            swapOutSet.add( cacheId )
            nToSwapOut -= 1
            if nToSwapOut <= 0:
                break
        swapOutKeySet = set()
        for theKey, thePair in self.m_cachedItems.iteritems():
            cacheId, theVal = thePair
            if cacheId in swapOutSet:
                swapOutKeySet.add( theKey )
                del self.m_cacheUpdateTime[cacheId]
        for theKey in swapOutKeySet:
            del self.m_cachedItems[theKey]
                
    def refresh_cache_id( self ):
        # refresh the cache id when the nMaxCacheId becomes too large.
        newCacheId = 0
        newCacheItems = {}
        newCacheUpdateTime = {}
        for theKey, thePair in self.m_cachedItems.itervalues():
            newCacheId += 1
            cacheId, theVal = thePair
            newCacheItems[theKey] = (newCacheId, theVal)
            newCacheUpdateTime[newCacheId] = self.m_cacheUpdateTime[cacheId]
        self.m_cachedItems = newCacheItems
        self.m_cacheUpdateTime = newCacheUpdateTime
        self.m_nMaxCacheId = newCacheId
        
    def __getitem__( self, theKey ):
        theVal = None
        if self.m_cachedItems.has_key( theKey ):
            self.m_nCacheUpdateThreshold -= 1
            cacheId, theVal = self.m_cachedItems[theKey]
            # update the UpdateTime
            theTime = self.m_cacheUpdateTime[cacheId]
            # bit 28 of theTime is the Reference bit
            theTime |= ( 1 << 28 )
            self.m_cacheUpdateTime[cacheId] = theTime
            if self.m_nCacheUpdateThreshold < 0:
                # update the cacheUpdateTime
                self.update_cache_info()
        else:  # cache miss
            # get the remote value
            theVal = self.m_rsHashClient.get( theKey )
            # check if we need to swap out
            if len( self.m_cachedItems ) >= self.m_nCacheSize:
                # write back and swap out
                self.do_swap_out()
            self.insert_item( theKey, theVal, False )
        return theVal

    def __setitem__( self, theKey, theVal ):
        if self.m_cachedItems.has_key( theKey ):
            cacheId, cacheVal = self.m_cachedItems[theKey]
            self.m_cachedItems[theKey] = ( cacheId, theVal )
            cacheVal = None
            theTime = self.m_cacheUpdateTime[cacheId]
            theTime |= ( 1 << 29 )  # update the Dirty bit
            theTime |= ( 1 << 28 )  # and the Reference bit
            self.m_cacheUpdateTime[cacheId] = theTime
            self.m_nCacheUpdateThreshold -= 1
        else: # the item is not in cache
            # swap out if need
            if len( self.m_cachedItems ) >= self.m_nCacheSize:
                self.do_swap_out()
            # insert it into the cache
            self.insert_item( theKey, theVal, True )

    def batch_set_item( self, itmList ):
        # this is for iterkeys and iteritems in iterator
        # When iterkeys is called, we also get the item itself because
        # there is a high probility that h[key] will be accessed
        for anItem in itmList:
            self.__setitem__( anItem[0], anItem[1] )
        
    def __delitem__( self, theKey ):
        bLocalItem = False
        if self.m_cachedItems.has_key( theKey ):
            cacheId, theVal = self.m_cachedItems[theKey]
            theTime = self.m_cacheUpdateTime[cacheId]
            bLocalItem = ( theTime & 0x40000000 ) 
            del self.m_cacheUpdateTime[cacheId]
            del self.m_cachedItems[theKey]
        if not bLocalItem:
            self.m_rsHashClient.remove( theKey )

    def has_key( self, theKey ):
        """
        Check if theKey is in this hash.

        Similar to dict.has_key
        """
        if self.m_cachedItems.has_key( theKey ):
            cacheId, theVal = self.m_cachedItems[theKey]
            theTime = self.m_cacheUpdateTime[cacheId]
            theTime |= ( 1 << 28 )
            self.m_cacheUpdateTime[cacheId] = theTime
            self.m_nCacheUpdateThreshold -= 1
            return True
        else:
            try:
                itm = self.__getitem__( theKey )
            except KeyError:
                return False
        return True

    def __len__( self ):
        remoteSize = self.m_rsHashClient.get_size()
        localSize = 0
        for aTime in self.m_cacheUpdateTime.itervalues():
            if (  aTime & 0x40000000  ):
                localSize += 1
        return ( remoteSize + localSize )

    def clear( self ):
        """
        Clear the hash.
        
        Similar to dict.clear()
        """
        self.m_rsHashClient.clear()
        self.m_cachedItems.clear()
        self.m_cacheUpdateTime.clear()
        self.m_nMaxCacheId = 0
        self.m_nCacheUpdateThreshold = self.m_nCacheSize / 28

    def iterkeys( self ):
        """
        Return the key iterator. This requires only a small amount of memory.

        The iterator is optimized with cache.
        """
        self.do_write_back()
        return MivRHashIter( 'keys', self )
    
    def itervalues( self ):
        """
        Return the value iterator. This requires only a small amount of memory.

        The iterator is optimized with cache.
        """
        self.do_write_back()
        return MivRHashIter( 'values', self )

    def iteritems( self ):
        """
        Return the item iterator. This requires only a small amount of memory.

        The iterator is optimized with cache.
        """
        self.do_write_back()
        return MivRHashIter( 'items', self )

    def __iter__( self ):
        self.do_write_back()
        return MivRHashIter( 'keys', self )

    def close( self ):
        """
        Free all memory on remote host and close the connection.
        """
        self.m_rsHashClient.close()

if __name__ == "__main__":
    print "MivRHash client testing ..."
    def rshashclient_main():
        hClient = RsHashClient()
        hClient.connect( "localhost", 10080 )
        i = 0
        while i < 1000000:
            hClient.put( i , i + 1 )
            if i % 10000 == 0:
                print i
            i += 1
        i = 0
        while i < 1000000:
            j = hClient.get( i )
            if i % 10000 == 0:
                print i
            if j != ( i + 1 ):
                raise Exception( "RsHashClient", "Invalid value", i )
            i += 1
        hClient = None
    rshashclient_main()



import random, time
if __name__ == '__main__':
    import os, sys
    sys.path.append( os.path.dirname( 
            os.path.dirname( os.path.abspath( __file__ ) ) ) )


from mivrhashclient import MivRHash


class MivRHashTest:
    def __init__( self ):
        self.m_h = MivRHash( "localhost", 10080, 2000 )

    def test_seq_insert( self ):
        print "Inserting 1000000 items ..."
        i = 0
        startTime = time.time()
        while i < 1000000:
            self.m_h[i] = i + 1
            i += 1
        finTime = time.time()
        elapsedTime = finTime - startTime
        print "Insert 1000000 items in %f [s]" % elapsedTime
        print "Hash size = %d" % len( self.m_h )
        
    def test_seq_access( self ):
        print "Accessing 1000000 items ..."
        i = 0
        startTime = time.time()
        while i < 1000000:
            j = self.m_h[i]
            if j != ( i + 1 ):
                raise Exception( "Invalid value at i = " % i )
            i += 1
        finTime = time.time()
        eTime = finTime - startTime
        print "Seq. accessed 1000000 items in %f [s]" % eTime

    def test_local_insert( self ):
        print "Local inserting 1000000 items ..."
        self.m_h.clear()
        i = 0
        startTime = time.time()
        while i < 1000000:
            k = i / 100000
            j = random.randint( k * 1000, k * 1000 + 999 )
            self.m_h[j] = j + 1
            i += 1
        finTime = time.time()
        eTime = finTime - startTime
        print "Local inserted 1000000 items in %f [s]" % eTime
        nSize = len( self.m_h )
        print "Hash size = %d" % nSize

    def test_local_access( self ):
        i = 0
        print "Local accessing 1000000 items ..."
        startTime = time.time()
        while i < 1000000:
            k = i / 100000
            j = random.randint( k * 1000, k * 1000 + 999 )
            if self.m_h.has_key( j ):
                if self.m_h[j] != ( j + 1 ):
                    raise Exception( "Invalid value at j = " % j )
            i += 1
        finTime = time.time()
        eTime = finTime - startTime
        print "Local accessed 1000000 items in %f [s]" % eTime
        
    def test_random_insert( self ):
        print "Random inserting 1000000 items ..."
        self.m_h.clear()
        i = 0
        startTime = time.time()
        while i < 1000000:
            # randomly insert with conflict keys
            j = random.randint( 0, 500000 )
            self.m_h[j] = j + 1
            i += 1
        finTime = time.time()
        eTime = finTime - startTime
        print "Random inserted 1000000 items in %f [s]" % eTime
        print "Hash size = %d" % len( self.m_h )

    def test_random_access( self ):
        print "Random accessing 1000000 items ..."
        i = 0
        startTime = time.time()
        while i < 1000000:
            j = random.randint( 0, 50000)
            if self.m_h.has_key( j ):
                if self.m_h[j] != ( j + 1 ):
                    raise Exception( "Invalid value at j = %d" % j )
            i += 1
            if i % 30000 == 0:
                print "i = %d" % i
        finTime = time.time()
        eTime = finTime - startTime
        print "Random accessed 1000000 items in %f [s]" % eTime

    def test_iterkeys( self ):
        print "Testing iterkeys"
        nCount = 0
        nSum = 0
        startTime = time.time()
        for i in self.m_h.iterkeys():
            nCount += 1
            nSum += i
            #assert( self.m_h[i] == ( i + 1 ) )
        finTime = time.time()
        eTime = finTime - startTime
        assert( nCount == 1000000 )
        assert( nSum == ( 1000000 / 2 ) * ( 1000000 - 1 ) )
        print "Iterkeys in %f [s]" % eTime

    def test_itervalues( self ):
        print "Testing itervalues"
        nCount = 0
        nSum = 0
        startTime = time.time()
        for i in self.m_h.itervalues():
            nCount += 1
            nSum += i
        finTime = time.time()
        eTime = finTime - startTime
        assert( nCount == 1000000 )
        assert( nSum == ( ( 1000000 / 2 ) * ( 1000000 + 1 ) ) )
        print "Itervalues in %f [s]" % eTime

    def test_iteritems( self ):
        print "Testing iteritems ..."
        nCount = 0
        nSum = 0
        startTime = time.time()
        for k, v in self.m_h.iteritems():
            nCount += 1
            nSum += k
            assert( v == ( k + 1 ) )
        finTime = time.time()
        eTime = finTime - startTime
        assert( nCount == 1000000 )
        assert( nSum == ( ( 1000000 / 2 ) * ( 1000000 - 1 ) ) )
        print "Iteritems in %f [s]" % eTime
        
    def run_test( self ):
        self.test_seq_insert()
        #self.test_seq_access()
        self.test_iterkeys()
        self.test_itervalues()
        self.test_iteritems()
        # self.test_local_insert()
        # self.test_local_access()
        # self.test_random_insert()
        # self.test_random_access()

if __name__ == '__main__':
    def speed_test_main():
        tCls = MivRHashTest()
        tCls.run_test()
        tCls.m_h = None

    speed_test_main()
